@@ -20,11 +20,19 @@ from typing import Dict
 from typing import List
 from typing import Optional
 
+from jinja2 import Environment, Undefined
+from jinja2 import PackageLoader
+
+from elyra.pipeline import pipeline_constants
+from elyra.pipeline.pipeline import KeyValueList
+from elyra.pipeline.pipeline import Operation
+
 
 class AppDataBase(object):  # ABC
     """
     An abstraction for app_data based nodes
     """
+
     _node: Dict = None
 
     def __init__(self, node: Dict):
@@ -40,7 +48,7 @@ class AppDataBase(object):  # ABC
         The node id
         :return: the node unique identifier
         """
-        return self._node.get('id')
+        return self._node.get("id")
 
     def get(self, key: str, default_value=None) -> Any:
         """
@@ -50,7 +58,7 @@ class AppDataBase(object):  # ABC
         :param default_value: a default value in case the key is not found
         :return: the value or the default_value if the key is not found
         """
-        return self._node['app_data'].get(key, default_value)
+        return self._node["app_data"].get(key, default_value)
 
     def set(self, key: str, value: Any):
         """
@@ -65,7 +73,7 @@ class AppDataBase(object):  # ABC
         if not value:
             raise ValueError("Value is required")
 
-        self._node['app_data'][key] = value
+        self._node["app_data"][key] = value
 
     def to_dict(self) -> Dict:
         return self._node
@@ -87,7 +95,7 @@ class Pipeline(AppDataBase):
         The pipeline version
         :return: The version
         """
-        return int(self._node['app_data'].get('version'))
+        return int(self._node["app_data"].get("version"))
 
     @property
     def runtime(self) -> str:
@@ -96,7 +104,7 @@ class Pipeline(AppDataBase):
         NOTE: This value should really be derived from runtime_config.
         :return: The runtime keyword
         """
-        return self._node['app_data'].get('runtime')
+        return self._node["app_data"].get("runtime")
 
     @property
     def runtime_config(self) -> str:
@@ -104,7 +112,7 @@ class Pipeline(AppDataBase):
 
         :return: The runtime configuration key. This should be a valid key from the Runtimes metadata
         """
-        return self._node['app_data'].get('runtime_config')
+        return self._node["app_data"].get("runtime_config")
 
     @property
     def type(self):
@@ -113,7 +121,7 @@ class Pipeline(AppDataBase):
         NOTE: This value should really be derived from runtime_config.
         :return: The runtime_type keyword associated with the pipeline.
         """
-        return self._node['app_data'].get('runtime_type')
+        return self._node["app_data"].get("runtime_type")
 
     @property
     def name(self) -> str:
@@ -121,7 +129,7 @@ class Pipeline(AppDataBase):
         The pipeline name
         :rtype: The pipeline name or `untitled`
         """
-        return self._node['app_data'].get('name', self._node['app_data'].get('properties', {}).get('name', 'untitled'))
+        return self._node["app_data"].get("name", self._node["app_data"].get("properties", {}).get("name", "untitled"))
 
     @property
     def source(self) -> str:
@@ -129,7 +137,7 @@ class Pipeline(AppDataBase):
         The pipeline source
         :rtype: The pipeline source
         """
-        return self._node['app_data'].get('source')
+        return self._node["app_data"].get("source")
 
     @property
     def nodes(self) -> list:
@@ -137,12 +145,12 @@ class Pipeline(AppDataBase):
         The list of nodes for the pipeline
         :rtype: object
         """
-        if 'nodes' not in self._node:
+        if "nodes" not in self._node:
             raise ValueError("Pipeline is missing 'nodes' field.")
 
         if self._nodes is None:
             nodes: list = list()
-            for node in self._node['nodes']:
+            for node in self._node["nodes"]:
                 nodes.append(Node(node))
 
             self._nodes = nodes
@@ -155,7 +163,25 @@ class Pipeline(AppDataBase):
         The list of user comments in the pipeline
         :rtype: list of comments
         """
-        return self._node['app_data']['ui_data'].get("comments", [])
+        return self._node["app_data"]["ui_data"].get("comments", [])
+
+    @property
+    def pipeline_parameters(self) -> Dict[str, Any]:
+        """
+        Retrieve pipeline parameters, which are defined as all
+        key/value pairs in the 'properties' stanza that are not
+        either pipeline meta-properties (e.g. name, description,
+        and runtime) or the pipeline defaults dictionary
+        """
+        all_properties = self._node["app_data"].get("properties", {})
+        excluded_properties = pipeline_constants.PIPELINE_META_PROPERTIES + [pipeline_constants.PIPELINE_DEFAULTS]
+
+        pipeline_parameters = {}
+        for property_name, value in all_properties.items():
+            if property_name not in excluded_properties:
+                pipeline_parameters[property_name] = value
+
+        return pipeline_parameters
 
     def get_property(self, key: str, default_value=None) -> Any:
         """
@@ -165,8 +191,8 @@ class Pipeline(AppDataBase):
         :return: the value or the default_value if the key is not found
         """
         return_value = default_value
-        if 'properties' in self._node['app_data']:
-            return_value = self._node['app_data']['properties'].get(key, default_value)
+        if "properties" in self._node["app_data"]:
+            return_value = self._node["app_data"]["properties"].get(key, default_value)
 
         return return_value
 
@@ -182,7 +208,23 @@ class Pipeline(AppDataBase):
         if not value:
             raise ValueError("Value is required")
 
-        self._node['app_data']['properties'][key] = value
+        self._node["app_data"]["properties"][key] = value
+
+    def convert_kv_properties(self, kv_properties: List[str]):
+        """
+        Convert pipeline defaults-level list properties that have been identified
+        as sets of key-value pairs from a plain list type to the KeyValueList type.
+        """
+        pipeline_defaults = self.get_property(pipeline_constants.PIPELINE_DEFAULTS, {})
+        for property_name, value in pipeline_defaults.items():
+            if property_name not in kv_properties:
+                continue
+
+            # Replace plain list with KeyValueList
+            pipeline_defaults[property_name] = KeyValueList(value)
+
+        if pipeline_defaults:
+            self.set_property(pipeline_constants.PIPELINE_DEFAULTS, pipeline_defaults)
 
 
 class Node(AppDataBase):
@@ -195,7 +237,7 @@ class Node(AppDataBase):
         The node type
         :return: type (e.g. execution_node, super_node)
         """
-        return self._node.get('type')
+        return self._node.get("type")
 
     @property
     def op(self) -> str:
@@ -203,7 +245,7 @@ class Node(AppDataBase):
         The node op, which identify the operation to be executed
         :return: op (e.g. execute-notebook-node)
         """
-        return self._node.get('op')
+        return self._node.get("op")
 
     @property
     def label(self) -> str:
@@ -211,7 +253,7 @@ class Node(AppDataBase):
         The node label
         :return:  node label
         """
-        return self._node['app_data']['ui_data'].get('label', self._node['app_data'].get('label', None))
+        return self._node["app_data"]["ui_data"].get("label", self._node["app_data"].get("label", None))
 
     @property
     def subflow_pipeline_id(self) -> Pipeline:
@@ -219,11 +261,11 @@ class Node(AppDataBase):
         The Super Node pipeline reference. Only available when type is a super node.
         :return:
         """
-        if self._node['type'] != 'super_node':
+        if self._node["type"] != "super_node":
             raise ValueError("Node must be a super_node in order to retrieve a subflow pipeline id")
 
-        if 'subflow_ref' in self._node:
-            return self._node['subflow_ref'].get('pipeline_id_ref')
+        if "subflow_ref" in self._node:
+            return self._node["subflow_ref"].get("pipeline_id_ref")
         else:
             return None
 
@@ -233,8 +275,8 @@ class Node(AppDataBase):
         Retrieve component links to other components.
         :return: the list of links associated with this node or an empty list if none are found
         """
-        if self.type in ['execution_node', 'super_node']:
-            return self._node['inputs'][0].get('links', [])
+        if self.type in ["execution_node", "super_node"]:
+            return self._node["inputs"][0].get("links", [])
         else:
             #  binding nodes do not contain links
             return []
@@ -245,8 +287,9 @@ class Node(AppDataBase):
         Retrieve the component source path.
         :return: None, if the node is a generic component, the component path otherwise.
         """
-        if self.type == 'execution_node':
-            return self._node['app_data'].get('component_source', None)
+        if self.type == "execution_node":
+            return self._node["app_data"].get("component_source", None)
+        return None
 
     def get_component_parameter(self, key: str, default_value=None) -> Any:
         """
@@ -256,7 +299,7 @@ class Node(AppDataBase):
         :param default_value: a default value in case the key is not found
         :return: the value or the default value if the key is not found
         """
-        value = self._node['app_data']['component_parameters'].get(key, default_value)
+        value = self._node["app_data"]["component_parameters"].get(key, default_value)
         return None if value == "None" else value
 
     def set_component_parameter(self, key: str, value: Any):
@@ -272,21 +315,46 @@ class Node(AppDataBase):
         if not value:
             raise ValueError("Value is required")
 
-        self._node['app_data']['component_parameters'][key] = value
+        self._node["app_data"]["component_parameters"][key] = value
+
+    def get_all_component_parameters(self) -> Dict[str, Any]:
+        """
+        Retrieve all component parameter key-value pairs.
+        """
+        return self._node["app_data"]["component_parameters"]
+
+    def convert_kv_properties(self, kv_properties: List[str]):
+        """
+        Convert node-level list properties that have been identified  as sets of
+        key-value pairs from a plain list type to the KeyValueList type.
+        """
+        for property_name, value in self.get_all_component_parameters().items():
+            if property_name not in kv_properties:
+                continue
+
+            if not value:
+                continue
+
+            # Convert plain list to KeyValueList
+            self.set_component_parameter(property_name, KeyValueList(value))
 
 
 class PipelineDefinition(object):
     """
     Represents a helper class to manipulate pipeline json structure
     """
+
     _pipelines: list = None
     _primary_pipeline: Pipeline = None
     _validated: bool = False
     _validation_issues: list = None
 
-    def __init__(self, pipeline_path: Optional[str] = None,
-                 pipeline_definition: Optional[Dict] = None,
-                 validate: bool = False):
+    def __init__(
+        self,
+        pipeline_path: Optional[str] = None,
+        pipeline_definition: Optional[Dict] = None,
+        validate: bool = False,
+    ):
         """
         The constructor enables either passing a pipeline path or the content of the pipeline definition.
         :param pipeline_path: this is the path to a pipeline
@@ -317,13 +385,15 @@ class PipelineDefinition(object):
         if validate:
             self.validate()
 
+        self.propagate_pipeline_default_properties()
+
     @property
     def id(self) -> str:
         """
         The pipeline definition id
         :return: the unid
         """
-        return self._pipeline_definition.get('id')
+        return self._pipeline_definition.get("id")
 
     @property
     def schema_version(self) -> str:
@@ -331,7 +401,7 @@ class PipelineDefinition(object):
         The schema used by the Pipeline definition
         :return: the version
         """
-        return self._pipeline_definition.get('version')
+        return self._pipeline_definition.get("version")
 
     @property
     def pipelines(self) -> list:
@@ -340,13 +410,13 @@ class PipelineDefinition(object):
         :return: the list of pipelines
         """
         if not self._pipelines:
-            if 'pipelines' not in self._pipeline_definition:
+            if "pipelines" not in self._pipeline_definition:
                 raise ValueError("Pipeline is missing 'pipelines' field.")
-            elif len(self._pipeline_definition['pipelines']) == 0:
+            elif len(self._pipeline_definition["pipelines"]) == 0:
                 raise ValueError("Pipeline has zero length 'pipelines' field.")
 
             pipelines: list = list()
-            for pipeline in self._pipeline_definition['pipelines']:
+            for pipeline in self._pipeline_definition["pipelines"]:
                 pipelines.append(Pipeline(pipeline))
 
             self._pipelines = pipelines
@@ -362,16 +432,23 @@ class PipelineDefinition(object):
         if not self._primary_pipeline:
             if "pipelines" not in self._pipeline_definition:
                 raise ValueError("Pipeline is missing 'pipelines' field.")
-            elif len(self._pipeline_definition['pipelines']) == 0:
+            elif len(self._pipeline_definition["pipelines"]) == 0:
                 raise ValueError("Pipeline has zero length 'pipelines' field.")
 
             # Find primary pipeline
-            self._primary_pipeline = self.get_pipeline_definition(self._pipeline_definition.get('primary_pipeline'))
+            self._primary_pipeline = self.get_pipeline_definition(self._pipeline_definition.get("primary_pipeline"))
 
             assert self._primary_pipeline is not None, "No primary pipeline was found"
 
         return self._primary_pipeline
 
+    @property
+    def pipeline_nodes(self) -> List[Node]:
+        """
+        All nodes of all pipelines associated with a pipeline definition
+        """
+        return [node for pipeline in self.pipelines for node in pipeline.nodes]
+
     def validate(self) -> list:
         """
         Validates the pipeline definition structure and semantics
@@ -385,50 +462,82 @@ class PipelineDefinition(object):
         # Has not been validated before
         validation_issues = list()
         # Validate pipeline schema version
-        if 'version' not in self._pipeline_definition:
+        if "version" not in self._pipeline_definition:
             validation_issues.append("Pipeline schema version field is missing.")
-        elif not isinstance(self._pipeline_definition['version'], str):
+        elif not isinstance(self._pipeline_definition["version"], str):
             validation_issues.append("Pipeline schema version field should be a string.")
 
         # Validate pipelines
-        if 'pipelines' not in self._pipeline_definition:
+        if "pipelines" not in self._pipeline_definition:
             validation_issues.append("Pipeline is missing 'pipelines' field.")
         elif not isinstance(self._pipeline_definition["pipelines"], list):
             validation_issues.append("Field 'pipelines' should be a list.")
-        elif len(self._pipeline_definition['pipelines']) == 0:
+        elif len(self._pipeline_definition["pipelines"]) == 0:
             validation_issues.append("Pipeline has zero length 'pipelines' field.")
 
         # Validate primary pipeline
-        if 'primary_pipeline' not in self._pipeline_definition:
+        if "primary_pipeline" not in self._pipeline_definition:
             validation_issues.append("Could not determine the primary pipeline.")
         elif not isinstance(self._pipeline_definition["primary_pipeline"], str):
             validation_issues.append("Field 'primary_pipeline' should be a string.")
 
-        primary_pipeline = self.get_pipeline_definition(self._pipeline_definition.get('primary_pipeline'))
+        primary_pipeline = self.get_pipeline_definition(self._pipeline_definition.get("primary_pipeline"))
         if not primary_pipeline:
             validation_issues.append("No primary pipeline was found")
         else:
             primary_pipeline = primary_pipeline.to_dict()
             # Validate primary pipeline structure
-            if 'app_data' not in primary_pipeline:
+            if "app_data" not in primary_pipeline:
                 validation_issues.append("Primary pipeline is missing the 'app_data' field.")
             else:
-                if 'version' not in primary_pipeline['app_data']:
+                if "version" not in primary_pipeline["app_data"]:
                     validation_issues.append("Primary pipeline is missing the 'version' field.")
-                if 'properties' not in primary_pipeline['app_data']:
+                if "properties" not in primary_pipeline["app_data"]:
                     validation_issues.append("Node is missing 'properties' field.")
-                elif len(primary_pipeline['app_data']['properties']) == 0:
+                elif len(primary_pipeline["app_data"]["properties"]) == 0:
                     validation_issues.append("Pipeline has zero length 'properties' field.")
 
-            if 'nodes' not in primary_pipeline or len(primary_pipeline['nodes']) == 0:
+            if "nodes" not in primary_pipeline or len(primary_pipeline["nodes"]) == 0:
                 validation_issues.append("At least one node must exist in the primary pipeline.")
             else:
-                for node in primary_pipeline['nodes']:
-                    if "component_parameters" not in node['app_data']:
+                for node in primary_pipeline["nodes"]:
+                    if "component_parameters" not in node["app_data"]:
                         validation_issues.append("Node is missing 'component_parameters' field")
 
         return validation_issues
 
+    def propagate_pipeline_default_properties(self):
+        """
+        For any default pipeline properties set (e.g. runtime image, volume), propagate
+        the values to any nodes that do not set their own value for that property.
+        """
+        # Convert any key-value list pipeline default properties to the KeyValueList type
+        kv_properties = PipelineDefinition.get_kv_properties()
+        self.primary_pipeline.convert_kv_properties(kv_properties)
+
+        pipeline_default_properties = self.primary_pipeline.get_property(pipeline_constants.PIPELINE_DEFAULTS, {})
+        for node in self.pipeline_nodes:
+            if not Operation.is_generic_operation(node.op):
+                continue
+
+            # Convert any key-value list node properties to the KeyValueList type
+            node.convert_kv_properties(kv_properties)
+
+            for property_name, pipeline_default_value in pipeline_default_properties.items():
+                if not pipeline_default_value:
+                    continue
+
+                node_value = node.get_component_parameter(property_name)
+                if not node_value:
+                    node.set_component_parameter(property_name, pipeline_default_value)
+                    continue
+
+                if isinstance(pipeline_default_value, KeyValueList):
+                    if not isinstance(node_value, KeyValueList):
+                        raise TypeError(f"The value of node property '{property_name}' is not of type KeyValueList")
+                    merged_list: KeyValueList = KeyValueList.merge(node_value, pipeline_default_value)
+                    node.set_component_parameter(property_name, merged_list)
+
     def is_valid(self) -> bool:
         """
         Represents whether or not the pipeline structure is valid
@@ -449,9 +558,9 @@ class PipelineDefinition(object):
         :param pipeline_id: the pipeline unique identifier
         :return: the pipeline or None
         """
-        if 'pipelines' in self._pipeline_definition:
+        if "pipelines" in self._pipeline_definition:
             for pipeline in self._pipeline_definition["pipelines"]:
-                if pipeline['id'] == pipeline_id:
+                if pipeline["id"] == pipeline_id:
                     return Pipeline(pipeline)
 
         return None
@@ -481,7 +590,7 @@ class PipelineDefinition(object):
             for comment in comment_list:
                 associated_node_id_list = comment.get("associated_id_refs", [])
                 for ref in associated_node_id_list:
-                    if ref['node_ref'] == node_id:
+                    if ref["node_ref"] == node_id:
                         comments.append(comment.get("content", ""))
 
         # remove empty (or whitespace-only) comment strings
@@ -503,3 +612,51 @@ class PipelineDefinition(object):
                 if node.type == "super_node":
                     supernode_list.append(node)
         return supernode_list
+
+    @staticmethod
+    def get_canvas_properties_from_template(package_name: str, template_name: str) -> Dict[str, Any]:
+        """
+        Retrieves the dict representation of the canvas-formatted properties
+        associated with the given template and package names. Rendering does
+        not require parameters as expressions are not evaluated due to the
+        SilentUndefined class.
+        """
+        loader = PackageLoader("elyra", package_name)
+        template_env = Environment(loader=loader, undefined=SilentUndefined)
+
+        template = template_env.get_template(template_name)
+        output = template.render()
+        return json.loads(output)
+
+    @staticmethod
+    def get_kv_properties() -> List[str]:
+        """
+        Get pipeline properties in its canvas form and loop through to
+        find those that should consist of key/value pairs, as given in
+        the 'keyValueEntries' key.
+        """
+        canvas_pipeline_properties = PipelineDefinition.get_canvas_properties_from_template(
+            package_name="templates/pipeline", template_name="pipeline_properties_template.jinja2"
+        )
+
+        kv_properties = []
+        parameter_info = canvas_pipeline_properties.get("uihints", {}).get("parameter_info", [])
+        for parameter in parameter_info:
+            if parameter.get("data", {}).get("keyValueEntries", False):
+                parameter_ref = parameter.get("parameter_ref", "")
+                if parameter_ref.startswith("elyra_"):
+                    parameter_ref = parameter_ref.replace("elyra_", "")
+                kv_properties.append(parameter_ref)
+
+        return kv_properties
+
+
+class SilentUndefined(Undefined):
+    """
+    A subclass of the jinja2.Undefined class used to represent undefined
+    values in the template. Undefined errors as a result of the evaluation
+    of expressions will fail silently and render as null.
+    """
+
+    def _fail_with_undefined_error(self, *args, **kwargs):
+        return None
